// Copyright 2002-2011, University of Colorado
/**
 * This class represents creates a graphical display that is meant to look
 * like a bicycle pump.  It allows the user to interact with it to move the
 * handle up and down.
 *
 * @author John Blanco
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var BasicStroke = require('java.awt.BasicStroke');
var Color = require( 'SCENERY/util/Color' );
var Cursor = require('java.awt.Cursor');
var GradientPaint = require('java.awt.GradientPaint');
var Rectangle = require('KITE/Rectangle');
var DoubleGeneralPath = require('edu.colorado.phet.common.phetcommon.view.util.DoubleGeneralPath');
var CursorHandler = require('edu.colorado.phet.common.piccolophet.event.CursorHandler');
var PhetPPath = require('edu.colorado.phet.common.piccolophet.nodes.PhetPPath');
var MultipleParticleModel = require('STATES_OF_MATTER/states-of-matter/model/MultipleParticleModel');
var Node = require('SCENERY/nodes/Node');
var PDragEventHandler = require('edu.umd.cs.piccolo.event.PDragEventHandler');
var PInputEvent = require('edu.umd.cs.piccolo.event.PInputEvent');
var PPath = require('edu.umd.cs.piccolo.nodes.PPath');
var PDimension = require('edu.umd.cs.piccolo.util.PDimension');
 
//------------------------------------------------------------------------
// Class Data
//------------------------------------------------------------------------
// The follow constants define the size and positions of the various
// components of the pump as proportions of the overall width and height
// of the node.
 
  //private
var PUMP_HORIZ_POSITION_PROPORTION = 0.85;
 
  //private
var PUMP_BASE_WIDTH_PROPORTION = 0.4;
 
  //private
var PUMP_BASE_HEIGHT_PROPORTION = 0.02;
 
  //private
var PUMP_BASE_COLOR = new Color(0xbb8855);
 
  //private
var PUMP_BODY_HEIGHT_PROPORTION = 0.75;
 
  //private
var PUMP_BODY_WIDTH_PROPORTION = 0.1;
 
  //private
var PUMP_BODY_COLOR = Color.red;
 
  //private
var PUMP_SHAFT_WIDTH_PROPORTION = PUMP_BODY_WIDTH_PROPORTION * 0.25;
 
  //private
var PUMP_SHAFT_HEIGHT_PROPORTION = PUMP_BODY_HEIGHT_PROPORTION;
 
  //private
var PUMP_SHAFT_COLOR = Color.LIGHT_GRAY;
 
  //private
var PUMP_HANDLE_WIDTH_PROPORTION = 0.35;
 
  //private
var PUMP_HANDLE_HEIGHT_PROPORTION = 0.02;
 
  //private
var PUMP_HANDLE_INIT_VERT_POS_PROPORTION = PUMP_BODY_HEIGHT_PROPORTION * 1.1;
 
  //private
var PUMP_HANDLE_COLOR = new Color(0xddaa77);
 
  //private
var HOSE_CONNECTOR_HEIGHT_PROPORTION = 0.04;
 
  //private
var HOSE_CONNECTOR_WIDTH_PROPORTION = 0.05;
 
  //private
var HOSE_CONNECTOR_VERT_POS_PROPORTION = 0.5;
 
  //private
var HOSE_CONNECTOR_COLOR = new Color(0xffff99);
 
  //private
var HOSE_ATTACH_VERT_POS_PROPORTION = 0.075;
 
  //private
var HOSE_COLOR = new Color(200, 200, 200);
 
  //private
var HOSE_WIDTH_PROPORTION = 0.03;
 
  //private
var PUMPING_REQUIRED_TO_INJECT_PROPORTION = PUMP_SHAFT_HEIGHT_PROPORTION / 10;
//------------------------------------------------------------------------
function BicyclePumpNode( width,  height,  model) {
    //------------------------------------------------------------------------
    // Instance Data
    //------------------------------------------------------------------------
    this.m_model;
    this.m_pumpHandle;
    this.m_currentHandleOffset;
    this.m_maxHandleOffset;
    this.m_pumpingRequiredToInject;
    this.m_currentPumpingAmount;
    this.m_pumpShaft;
    // Initialize local variables.
    m_model = model;
    m_pumpingRequiredToInject = height * PUMPING_REQUIRED_TO_INJECT_PROPORTION;
    m_currentPumpingAmount = 0;
    // Add the base of the pump.
    var pumpBaseWidth = width * PUMP_BASE_WIDTH_PROPORTION;
    var pumpBaseHeight = height * PUMP_BASE_HEIGHT_PROPORTION;
    var pumpBase = new PPath(new Rectangle.Number(0, 0, pumpBaseWidth, pumpBaseHeight));
    pumpBase.setPaint(PUMP_BASE_COLOR);
    pumpBase.setOffset(width * PUMP_HORIZ_POSITION_PROPORTION - (pumpBaseWidth / 2), height - pumpBaseHeight);
    pumpBase.setPickable(false);
    addChild(pumpBase);
    // interact with in order to use the pump.
    var pumpHandleWidth = width * PUMP_HANDLE_WIDTH_PROPORTION;
    var pumpHandleHeight = height * PUMP_HANDLE_HEIGHT_PROPORTION;
    m_pumpHandle = new PPath(new Rectangle.Number(0, 0, pumpHandleWidth, pumpHandleHeight));
    m_pumpHandle.setPaint(PUMP_HANDLE_COLOR);
    m_pumpHandle.setOffset(width * PUMP_HORIZ_POSITION_PROPORTION - (pumpHandleWidth / 2), height - (height * PUMP_HANDLE_INIT_VERT_POS_PROPORTION) - pumpHandleHeight);
    m_pumpHandle.setPickable(true);
    m_pumpHandle.addInputEventListener(new CursorHandler(Cursor.N_RESIZE_CURSOR));
    m_currentHandleOffset = 0;
    m_maxHandleOffset = -PUMP_SHAFT_HEIGHT_PROPORTION * height / 2;
    addChild(m_pumpHandle);
    // the handle.
    m_pumpHandle.addInputEventListener(new PDragEventHandler().withAnonymousClassBody( {
        startDrag: function( event) {
            super.startDrag(event);
        },
        drag: function( event) {
            var d = event.getDeltaRelativeTo(m_pumpHandle);
            m_pumpHandle.localToParent(d);
            if ((m_currentHandleOffset + d.getHeight() >= m_maxHandleOffset) && (m_currentHandleOffset + d.getHeight() <= 0)) {
                m_pumpHandle.offset(0, d.getHeight());
                m_pumpShaft.offset(0, d.getHeight());
                m_currentHandleOffset += d.getHeight();
                if (d.getHeight() > 0) {
                    // This motion is in the pumping direction, so accumulate it.
                    m_currentPumpingAmount += d.getHeight();
                    if (m_currentPumpingAmount >= m_pumpingRequiredToInject) {
                        // Enough pumping has been done to inject a new particle.
                        m_model.injectMolecule();
                        m_currentPumpingAmount = 0;
                    }
                }
            }
        }
    }));
    // Add the shaft for the pump.
    var pumpShaftWidth = width * PUMP_SHAFT_WIDTH_PROPORTION;
    var pumpShaftHeight = height * PUMP_SHAFT_HEIGHT_PROPORTION;
    m_pumpShaft = new PPath(new Rectangle.Number(0, 0, pumpShaftWidth, pumpShaftHeight));
    m_pumpShaft.setPaint(PUMP_SHAFT_COLOR);
    m_pumpShaft.setOffset(width * PUMP_HORIZ_POSITION_PROPORTION - (pumpShaftWidth / 2), height - (height * PUMP_HANDLE_INIT_VERT_POS_PROPORTION));
    m_pumpShaft.setPickable(false);
    addChild(m_pumpShaft);
    // Add the hose.
    var hoseExternalAttachPtX = 0;
    var hoseExternalAttachPtY = height - (height * HOSE_CONNECTOR_VERT_POS_PROPORTION) + (height * HOSE_CONNECTOR_HEIGHT_PROPORTION / 2);
    var hoseToPumpAttachPtX = width * PUMP_HORIZ_POSITION_PROPORTION;
    var hoseToPumpAttachPtY = height - (height * HOSE_ATTACH_VERT_POS_PROPORTION);
    var hosePath = new DoubleGeneralPath();
    var hoseWidth = HOSE_WIDTH_PROPORTION * width;
    hosePath.moveTo(hoseExternalAttachPtX, hoseExternalAttachPtY + hoseWidth / 2);
    hosePath.lineTo(hoseExternalAttachPtX, hoseExternalAttachPtY - hoseWidth / 2);
    hosePath.curveTo(width + hoseWidth, height - (height * HOSE_CONNECTOR_VERT_POS_PROPORTION) - hoseWidth / 2, hoseWidth, height - (height * HOSE_ATTACH_VERT_POS_PROPORTION) - hoseWidth / 2, hoseToPumpAttachPtX, hoseToPumpAttachPtY - hoseWidth / 2);
    hosePath.lineTo(hoseToPumpAttachPtX, hoseToPumpAttachPtY + hoseWidth / 2);
    hosePath.curveTo(-hoseWidth, height - (height * HOSE_ATTACH_VERT_POS_PROPORTION) + hoseWidth / 2, width, height - (height * HOSE_CONNECTOR_VERT_POS_PROPORTION) + hoseWidth / 2, hoseExternalAttachPtX, hoseExternalAttachPtY + hoseWidth / 2);
    hosePath.closePath();
    var hose = new PhetPPath(hosePath.getGeneralPath(), HOSE_COLOR, new BasicStroke(20), Color.BLACK);
    hose.setPickable(false);
    addChild(hose);
    // Add the hose connector.
    var hoseConnectorWidth = width * HOSE_CONNECTOR_WIDTH_PROPORTION;
    var hoseConnectorHeight = height * HOSE_CONNECTOR_HEIGHT_PROPORTION;
    var hoseConnector = new PPath(new Rectangle.Number(0, 0, hoseConnectorWidth, hoseConnectorHeight));
    hoseConnector.setPaint(HOSE_CONNECTOR_COLOR);
    hoseConnector.setOffset(0, height - (height * HOSE_CONNECTOR_VERT_POS_PROPORTION));
    hoseConnector.setPickable(false);
    addChild(hoseConnector);
    // Add the body of the pump
    var pumpBodyWidth = width * PUMP_BODY_WIDTH_PROPORTION;
    var pumpBodyHeight = height * PUMP_BODY_HEIGHT_PROPORTION;
    var pumpBody = new PhetPPath(new Rectangle.Number(0, 0, pumpBodyWidth, pumpBodyHeight));
    var pumpBodyPaint = new GradientPaint(0, pumpBodyHeight / 2, PUMP_BODY_COLOR, pumpBodyWidth, pumpBodyHeight / 2, Color.LIGHT_GRAY);
    pumpBody.setPaint(pumpBodyPaint);
    pumpBody.setOffset(width * PUMP_HORIZ_POSITION_PROPORTION - (pumpBodyWidth / 2), height - pumpBodyHeight - pumpBaseHeight);
    pumpBody.setPickable(false);
    addChild(pumpBody);
}
  
  return inherit(Node,BicyclePumpNode,{
});
} );

