// Copyright 2002-2011, University of Colorado
/**
 * This extension of the ParticleNode class allows users to grab the node and
 * move it, thus changing the position within the underlying model.
 *
 * @author John Blanco
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var Cursor = require('java.awt.Cursor');
var CursorHandler = require('edu.colorado.phet.common.piccolophet.event.CursorHandler');
var DualAtomModel = require('STATES_OF_MATTER/states-of-matter/model/DualAtomModel');
var StatesOfMatterAtom = require('STATES_OF_MATTER/states-of-matter/model/particle/StatesOfMatterAtom');
var Node = require('SCENERY/nodes/Node');
var PDragEventHandler = require('edu.umd.cs.piccolo.event.PDragEventHandler');
var PInputEvent = require('edu.umd.cs.piccolo.event.PInputEvent');
var PDimension = require('edu.umd.cs.piccolo.util.PDimension');
 
/**
     * Constructor.
     *
     * @param particle      - The particle within the model.
     * @param mvt           - Model-view transform.
     * @param enableOverlap TODO
     * @param minX          - Minimum value in the X direction to which the particle can be moved.
     * @param maxX          - Maximum value in the X direction to which the particle can be moved.
     */
function GrabbableParticleNode( model,  particle,  mvt,  useGradient,  enableOverlap,  minX,  maxX) {
    //----------------------------------------------------------------------------
    // Class Data
    //----------------------------------------------------------------------------
    //----------------------------------------------------------------------------
    // Instance Data
    //----------------------------------------------------------------------------
    this.m_model;
    // The minimum value allowed in the x direction.
    this.m_minX;
    // The minimum value allowed in the x direction.
    this.m_maxX;
    ParticleForceNode.call(this, particle, mvt, useGradient, enableOverlap);
    m_model = model;
    m_minX = minX;
    m_maxX = maxX;
    // This node will need to be pickable so the user can grab it.
    setPickable(true);
    setChildrenPickable(true);
    // Put a cursor handler into place.
    addInputEventListener(new CursorHandler(Cursor.HAND_CURSOR));
    addInputEventListener(new PDragEventHandler().withAnonymousClassBody( {
        startDrag: function( event) {
            super.startDrag(event);
            handleMouseStartDragEvent();
        },
        drag: function( event) {
            handleMouseDragEvent(event);
        },
        endDrag: function( event) {
            super.endDrag(event);
            handleMouseEndDragEvent();
        }
    }));
}
  
  return inherit(ParticleForceNode,GrabbableParticleNode,{
//----------------------------------------------------------------------------
// Accessor Methods
//----------------------------------------------------------------------------
getMinX: function() {
    return m_minX;
},
setMinX: function( minX) {
    m_minX = minX;
},
getMaxX: function() {
    return m_maxX;
},
setMaxX: function( maxX) {
    m_maxX = maxX;
},
//----------------------------------------------------------------------------
// Private Methods
//----------------------------------------------------------------------------
 
  //private
handleMouseStartDragEvent: function() {
    // is moving it.
    m_model.setMotionPaused(true);
},
 
  //private
handleMouseDragEvent: function( event) {
    // Only allow the user to move unbonded atoms.
    if (m_model.getBondingState() != DualAtomModel.BONDING_STATE_UNBONDED) {
        // Need to release the bond before we can move the atom.
        m_model.releaseBond();
    }
    // Determine the amount of motion indicated by the drag event.
    var draggedNode = event.getPickedNode();
    var d = event.getDeltaRelativeTo(draggedNode);
    draggedNode.localToParent(d);
    // Make sure we don't exceed the positional limits.
    var newPosX = m_particle.getX() + d.width;
    if (newPosX > m_maxX) {
        newPosX = m_maxX;
    } else if (newPosX < m_minX) {
        newPosX = m_minX;
    }
    // Move the particle based on the amount of mouse movement.
    m_particle.setPosition(newPosX, m_particle.getY());
},
 
  //private
handleMouseEndDragEvent: function() {
    // even if the motion was paused by some other means.
    m_model.setMotionPaused(false);
}
});
} );

