// Copyright 2002-2011, University of Colorado
/**
 * This class is the "view" for the particle container.  This is where the
 * information about the nature of the image that is used to depict the
 * container is encapsulated.
 *
 * @author John Blanco
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var Color = require( 'SCENERY/util/Color' );
var Rectangle = require('KITE/Rectangle');
var Random = require('java.util.Random');
var VoidFunction1 = require('edu.colorado.phet.common.phetcommon.util.function.VoidFunction1');
var PhetPNode = require('edu.colorado.phet.common.piccolophet.PhetPNode');
var HandleNode = require('edu.colorado.phet.common.piccolophet.nodes.HandleNode');
var StatesOfMatterConstants = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterConstants');
var StatesOfMatterGlobalState = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterGlobalState');
var StatesOfMatterResources = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterResources');
var StatesOfMatterStrings = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterStrings');
var MultipleParticleModel = require('STATES_OF_MATTER/states-of-matter/model/MultipleParticleModel');
var HydrogenAtom = require('STATES_OF_MATTER/states-of-matter/model/particle/HydrogenAtom');
var StatesOfMatterAtom = require('STATES_OF_MATTER/states-of-matter/model/particle/StatesOfMatterAtom');
var DialGaugeNode = require('STATES_OF_MATTER/states-of-matter/view/instruments/DialGaugeNode');
var Node = require('SCENERY/nodes/Node');
var PImage = require('edu.umd.cs.piccolo.nodes.PImage');
 
//----------------------------------------------------------------------------
// Class Data
//----------------------------------------------------------------------------
var LOAD_CONTAINER_BACKGROUND_IMAGE = false;
// Constants that control the appearance of the image.
 
  //private
var CONTAINER_FRONT_LEFT_IMAGE_NAME = "cup_3D_front_70_split_left.png";
 
  //private
var CONTAINER_FRONT_RIGHT_IMAGE_NAME = "cup_3D_front_70_split_right.png";
 
  //private
var CONTAINER_FRONT_TOP_IMAGE_NAME = "cup_3D_front_70_split_top.png";
 
  //private
var CONTAINER_FRONT_BOTTOM_IMAGE_NAME = "cup_3D_front_70_split_bottom.png";
 
  //private
var CONTAINER_BACK_TOP_IMAGE_NAME = "cup_3D_top_70_fragment.png";
 
  //private
var LID_IMAGE_NAME = "cup_3D_cap_70.png";
 
  //private
var CONTAINER_BACK_IMAGE_NAME = "cup_3D_back_light.jpg";
// Empirically determined value for aligning lid and container body.
 
  //private
var LID_POSITION_TWEAK_FACTOR = 65;
// Empirically determined value for position corresponding fragment.
 
  //private
var FRONT_TOP_IMAGE_OFFSET_TWEAK = 19;
// Empirically determined value for position corresponding fragment.
 
  //private
var FRONT_BOTTOM_IMAGE_OFFSET_TWEAK = 17;
// Constant(s) that affect the appearance of both depictions of the container.
// Height of ellipses as a function of overall height.
 
  //private
var ELLIPSE_HEIGHT_PROPORTION = 0.15;
 
  //private
var PRESSURE_GAUGE_WIDTH_PROPORTION = 0.44;
// Constants the control the positioning of non-container portions of this node.
 
  //private
var PRESSURE_GAUGE_Y_OFFSET = -3000;
 
  //private
var PRESSURE_METER_X_OFFSET_PROPORTION = 0.80;
// Maximum value expected for pressure, in atmospheres.
 
  //private
var MAX_PRESSURE = 200;
function ParticleContainerNode( model,  mvt,  volumeControlEnabled,  pressureGaugeEnabled) {
    //----------------------------------------------------------------------------
    // Instance Data
    //----------------------------------------------------------------------------
     
  //private
this.m_model;
     
  //private
this.m_mvt;
     
  //private
this.m_containmentAreaWidth;
     
  //private
this.m_containmentAreaHeight;
     
  //private
this.m_lowerParticleLayer;
     
  //private
this.m_upperParticleLayer;
     
  //private
this.m_topContainerLayer;
     
  //private
this.m_middleContainerLayer;
     
  //private
this.m_bottomContainerLayer;
     
  //private
this.m_containerLid;
     
  //private
this.m_rand;
     
  //private
this.m_rotationAmount;
     
  //private
this.m_pressureMeter;
     
  //private
this.m_pressureMeterElbowOffset;
     
  //private
this.m_containerHeightAtExplosion;
    m_model = model;
    m_mvt = mvt;
    m_containmentAreaWidth = StatesOfMatterConstants.CONTAINER_BOUNDS.getWidth();
    m_containmentAreaHeight = StatesOfMatterConstants.CONTAINER_BOUNDS.getHeight();
    m_rand = new Random();
    // Set ourself up as a listener to the model.
    m_model.addListener(new MultipleParticleModel.Adapter().withAnonymousClassBody( {
        particleAdded: function( particle) {
            var particleNode = new ParticleNode(particle, m_mvt);
            if (particle instanceof HydrogenAtom) {
                // so some will end up on the top layer.
                m_lowerParticleLayer.addChild(particleNode);
            } else {
                m_upperParticleLayer.addChild(particleNode);
            }
            // set this way.
            var whiteBackgroundObserver = new VoidFunction1().withAnonymousClassBody( {
                apply: function( whiteBackground) {
                    particleNode.setStrokeEnabled(whiteBackground);
                }
            });
            StatesOfMatterGlobalState.whiteBackground.addObserver(whiteBackgroundObserver);
            // particle goes away.
            particle.addListener(new StatesOfMatterAtom.Adapter().withAnonymousClassBody( {
                particleRemoved: function( particle) {
                    StatesOfMatterGlobalState.whiteBackground.removeObserver(whiteBackgroundObserver);
                }
            }));
        },
        containerSizeChanged: function() {
            handleContainerSizeChanged();
        },
        pressureChanged: function() {
            if (m_pressureMeter != null) {
                m_pressureMeter.setValue(m_model.getPressureInAtmospheres());
            }
        },
        containerExplodedStateChanged: function( containerExploded) {
            m_containerHeightAtExplosion = m_model.getParticleContainerHeight();
            m_rotationAmount = Math.PI / 100 + (m_rand.nextDouble() * Math.PI / 50);
            if (m_rand.nextBoolean()) {
                m_rotationAmount = -m_rotationAmount;
            }
        }
    }));
    // various components of this node can be added appropriately.
    m_bottomContainerLayer = new Node();
    m_bottomContainerLayer.setPickable(false);
    m_bottomContainerLayer.setChildrenPickable(false);
    addChild(m_bottomContainerLayer);
    m_lowerParticleLayer = new Node();
    m_lowerParticleLayer.setPickable(false);
    m_lowerParticleLayer.setChildrenPickable(false);
    // Compensate for inverted Y axis.
    m_lowerParticleLayer.setOffset(0, m_containmentAreaHeight);
    addChild(m_lowerParticleLayer);
    m_upperParticleLayer = new Node();
    m_upperParticleLayer.setPickable(false);
    m_upperParticleLayer.setChildrenPickable(false);
    // Compensate for inverted Y axis.
    m_upperParticleLayer.setOffset(0, m_containmentAreaHeight);
    addChild(m_upperParticleLayer);
    m_middleContainerLayer = new Node();
    m_middleContainerLayer.setPickable(false);
    m_middleContainerLayer.setChildrenPickable(true);
    addChild(m_middleContainerLayer);
    m_topContainerLayer = new Node();
    m_topContainerLayer.setPickable(false);
    m_topContainerLayer.setChildrenPickable(false);
    addChild(m_topContainerLayer);
    // Create the visual representation of the container.
    loadMultipleContainerImages();
    if (volumeControlEnabled) {
        // Add the finger for pressing down on the top of the container.
        var fingerNode = new PointingHandNode(m_model);
        // responsible for positioning itself later based on user interaction.
        m_middleContainerLayer.addChild(fingerNode);
        // Add the handle to the lid.
        var handleNode = new HandleNode(50, 100, Color.YELLOW);
        handleNode.rotate(Math.PI / 2);
        handleNode.setOffset((m_containerLid.getWidth() / 2) + (handleNode.getFullBoundsReference().width / 2), 0);
        m_containerLid.addChild(handleNode);
    }
    if (pressureGaugeEnabled) {
        // Add the pressure meter.
        m_pressureMeter = new DialGaugeNode(PRESSURE_GAUGE_WIDTH_PROPORTION * m_containmentAreaWidth, StatesOfMatterStrings.PRESSURE_GAUGE_TITLE, 0, MAX_PRESSURE, StatesOfMatterStrings.PRESSURE_GAUGE_UNITS);
        m_pressureMeter.setElbowEnabled(true);
        m_middleContainerLayer.addChild(m_pressureMeter);
        updatePressureGauge();
        m_pressureMeterElbowOffset = m_pressureMeter.getFullBoundsReference().getCenterY();
    }
    // x=0, y=0, is at the lower left corner of the container.
    var xPos = 0;
    var yPos = -m_containmentAreaHeight;
    setOffset(xPos, yPos);
    // Set ourself to be non-pickable so that we don't get mouse events.
    setPickable(false);
}
  
  return inherit(PhetPNode,ParticleContainerNode,{
//----------------------------------------------------------------------------
// Public Methods
//----------------------------------------------------------------------------
reset: function() {
},
//----------------------------------------------------------------------------
// Private Methods
//----------------------------------------------------------------------------
/**
     * Handle a notification that the container size has changed.
     */
 
  //private
handleContainerSizeChanged: function() {
    // changed.
    var containerHeight = m_model.getParticleContainerHeight();
    if (!m_model.getContainerExploded()) {
        if (m_containerLid.getRotation() != 0) {
            m_containerLid.setRotation(0);
        }
        m_containerLid.setOffset((m_containmentAreaWidth - m_containerLid.getFullBoundsReference().width) / 2, m_containmentAreaHeight - containerHeight - (m_containerLid.getFullBoundsReference().height / 2) + LID_POSITION_TWEAK_FACTOR);
    } else {
        // blown off the top of the container.
        m_containerLid.rotateAboutPoint(m_rotationAmount, (m_containmentAreaWidth / 2) / m_containerLid.getScale(), 0);
        var centerPosY = m_containmentAreaHeight - containerHeight - (m_containerLid.getFullBoundsReference().height / 2) + LID_POSITION_TWEAK_FACTOR;
        var currentPosY = m_containerLid.getOffset().getY();
        var newPosX = m_containerLid.getOffset().getX();
        var newPosY;
        if (currentPosY > centerPosY) {
            newPosY = centerPosY;
        } else {
            newPosY = currentPosY;
        }
        m_containerLid.setOffset(newPosX, newPosY);
    }
    updatePressureGauge();
},
 
  //private
loadMultipleContainerImages: function() {
    // Load the images that will make up the container.
    var containerLeftSideImageNode = StatesOfMatterResources.getImageNode(CONTAINER_FRONT_LEFT_IMAGE_NAME);
    var containerRightSideImageNode = StatesOfMatterResources.getImageNode(CONTAINER_FRONT_RIGHT_IMAGE_NAME);
    var containerBottomImageNode = StatesOfMatterResources.getImageNode(CONTAINER_FRONT_BOTTOM_IMAGE_NAME);
    var containerTopImageNode = StatesOfMatterResources.getImageNode(CONTAINER_FRONT_TOP_IMAGE_NAME);
    // Create a single PNode to contain these images, and add the images to it.
    var containerNode = new Node();
    containerNode.addChild(containerLeftSideImageNode);
    containerNode.addChild(containerTopImageNode);
    containerTopImageNode.setOffset(containerLeftSideImageNode.getFullBoundsReference().width, FRONT_TOP_IMAGE_OFFSET_TWEAK);
    containerNode.addChild(containerBottomImageNode);
    containerBottomImageNode.setOffset(containerLeftSideImageNode.getFullBoundsReference().width, containerLeftSideImageNode.getFullBoundsReference().height - containerBottomImageNode.getFullBoundsReference().height + FRONT_BOTTOM_IMAGE_OFFSET_TWEAK);
    containerNode.addChild(containerRightSideImageNode);
    containerRightSideImageNode.setOffset(containerBottomImageNode.getFullBoundsReference().getMaxX(), 0);
    // Scale the container node based on the size of the container.
    containerNode.setScale(m_containmentAreaWidth / containerNode.getFullBoundsReference().width);
    // Add the image to the top layer node.
    m_topContainerLayer.addChild(containerNode);
    containerNode.setOffset(0, 0);
    // Add the lid of the container.
    m_containerLid = StatesOfMatterResources.getImageNode(LID_IMAGE_NAME);
    m_containerLid.setScale(m_containmentAreaWidth / m_containerLid.getFullBoundsReference().width);
    m_containerLid.setPickable(false);
    m_middleContainerLayer.addChild(m_containerLid);
    m_containerLid.setOffset(0, (-m_containerLid.getFullBoundsReference().height / 2) + LID_POSITION_TWEAK_FACTOR);
    if (LOAD_CONTAINER_BACKGROUND_IMAGE) {
        // Load the image that will be used for the back of the container.
        var containerBackImageNode = StatesOfMatterResources.getImageNode(CONTAINER_BACK_IMAGE_NAME);
        // Scale the container image based on the size of the container.
        containerBackImageNode.setScale(m_containmentAreaWidth / containerBackImageNode.getFullBoundsReference().width);
        // Add the image to the bottom layer node.
        m_bottomContainerLayer.addChild(containerBackImageNode);
        containerBackImageNode.setOffset(0, -(m_model.getParticleContainerHeight() * ELLIPSE_HEIGHT_PROPORTION / 2));
    } else {
        // Just load an image for the back of the top.
        var containerTopBackImageNode = StatesOfMatterResources.getImageNode(CONTAINER_BACK_TOP_IMAGE_NAME);
        // Scale the container image based on the size of the container.
        containerTopBackImageNode.setScale(m_containmentAreaWidth / containerTopBackImageNode.getFullBoundsReference().width);
        // Add the image to the bottom layer node.
        m_bottomContainerLayer.addChild(containerTopBackImageNode);
        containerTopBackImageNode.setOffset(0, -(m_model.getParticleContainerHeight() * ELLIPSE_HEIGHT_PROPORTION / 2));
    }
},
/**
     * Update the position and other aspects of the gauge so that it stays
     * connected to the lid or moves as it should when the container
     * explodes.
     */
 
  //private
updatePressureGauge: function() {
    if (m_pressureMeter != null) {
        var containerRect = m_model.getParticleContainerRect();
        if (!m_model.getContainerExploded()) {
            if (m_pressureMeter.getRotation() != 0) {
                m_pressureMeter.setRotation(0);
            }
            m_pressureMeter.setOffset(-m_pressureMeter.getFullBoundsReference().width * PRESSURE_METER_X_OFFSET_PROPORTION, PRESSURE_GAUGE_Y_OFFSET);
            m_pressureMeter.setElbowHeight(StatesOfMatterConstants.PARTICLE_CONTAINER_INITIAL_HEIGHT - containerRect.getHeight() - m_pressureMeterElbowOffset);
        } else {
            // The container is exploding, so spin and move the gauge.
            m_pressureMeter.rotateInPlace(-Math.PI / 20);
            m_pressureMeter.setOffset(-m_pressureMeter.getFullBoundsReference().width * PRESSURE_METER_X_OFFSET_PROPORTION, PRESSURE_GAUGE_Y_OFFSET - m_model.getParticleContainerHeight() + m_containerHeightAtExplosion);
        }
    }
}
},
//statics
{
LOAD_CONTAINER_BACKGROUND_IMAGE:LOAD_CONTAINER_BACKGROUND_IMAGE
});
} );

