// Copyright 2002-2011, University of Colorado
/**
 * This class adds the ability to display force-depicting arrows to its super
 * class.
 *
 * @author John Blanco
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var Color = require( 'SCENERY/util/Color' );
var Vector2DNode = require('edu.colorado.phet.common.piccolophet.nodes.Vector2DNode');
var StatesOfMatterAtom = require('STATES_OF_MATTER/states-of-matter/model/particle/StatesOfMatterAtom');
 
//-----------------------------------------------------------------------------
// Class Data
//-----------------------------------------------------------------------------
// The following constants control some of the aspects of the appearance of
// the force arrows.  The values are arbitrary and are chosen to look good
// in this particular sim, so tweak them as needed for optimal appearance.
var ATTRACTIVE_FORCE_COLOR = new Color(255, 255, 0, 175);
//    public static final Color REPULSIVE_FORCE_COLOR = new Color( 255, 200, 0, 175 );
//    public static final Color REPULSIVE_FORCE_COLOR = new Color( 64, 224, 208, 175 );  // Cyanish, but a little too much like the initial atom color.
//    public static final Color REPULSIVE_FORCE_COLOR = new Color( 199, 21, 133, 175 );  // Purpleish, not bad, a little dark IMO
//    public static final Color REPULSIVE_FORCE_COLOR = new Color( 255, 105, 180, 175 ); // Hot pink. Not bad.  Looks kind of purple when transparent.
//    public static final Color REPULSIVE_FORCE_COLOR = new Color( 255, 182, 193, 175 ); // Light pink. Not bad. Looks kind of purple when transparent.
//    public static final Color REPULSIVE_FORCE_COLOR = PhetColorScheme.RED_COLORBLIND;
//    public static final Color REPULSIVE_FORCE_COLOR = new Color( 222, 184, 135, 175 );
//    public static final Color REPULSIVE_FORCE_COLOR = Color.WHITE;
// Magenta.
var REPULSIVE_FORCE_COLOR = new Color(255, 0, 255, 175);
var TOTAL_FORCE_COLOR = new Color(0, 255, 0, 125);
 
  //private
var COMPONENT_FORCE_ARROW_REFERENCE_LENGTH = 500;
 
  //private
var COMPONENT_FORCE_ARROW_REFERENCE_MAGNITUDE = 1E-22;
 
  //private
var TOTAL_FORCE_ARROW_REFERENCE_LENGTH = 1000;
 
  //private
var TOTAL_FORCE_ARROW_REFERENCE_MAGNITUDE = 1E-22;
 
  //private
var FORCE_ARROW_TAIL_WIDTH = 100;
 
  //private
var FORCE_ARROW_HEAD_WIDTH = 200;
 
  //private
var FORCE_ARROW_HEAD_LENGTH = 200;
function ParticleForceNode( particle,  mvt,  useGradient,  enableOverlap) {
    //-----------------------------------------------------------------------------
    // Instance Data
    //-----------------------------------------------------------------------------
     
  //private
this.m_attractiveForce;
     
  //private
this.m_attractiveForceVectorNode;
     
  //private
this.m_repulsiveForce;
     
  //private
this.m_repulsiveForceVectorNode;
     
  //private
this.m_totalForceVectorNode;
    ParticleNode.call(this, particle, mvt, useGradient, false, enableOverlap);
    m_attractiveForce = 0;
    m_repulsiveForce = 0;
    m_attractiveForceVectorNode = new Vector2DNode(0, 0, COMPONENT_FORCE_ARROW_REFERENCE_MAGNITUDE, COMPONENT_FORCE_ARROW_REFERENCE_LENGTH);
    m_attractiveForceVectorNode.setMagnitudeAngle(0, 0);
    addChild(m_attractiveForceVectorNode);
    m_attractiveForceVectorNode.setArrowFillPaint(ATTRACTIVE_FORCE_COLOR);
    m_attractiveForceVectorNode.setHeadSize(FORCE_ARROW_HEAD_WIDTH, FORCE_ARROW_HEAD_LENGTH);
    m_attractiveForceVectorNode.setTailWidth(FORCE_ARROW_TAIL_WIDTH);
    m_attractiveForceVectorNode.setVisible(false);
    m_repulsiveForceVectorNode = new Vector2DNode(0, 0, COMPONENT_FORCE_ARROW_REFERENCE_MAGNITUDE, COMPONENT_FORCE_ARROW_REFERENCE_LENGTH);
    m_repulsiveForceVectorNode.setMagnitudeAngle(0, 0);
    addChild(m_repulsiveForceVectorNode);
    m_repulsiveForceVectorNode.setArrowFillPaint(REPULSIVE_FORCE_COLOR);
    m_repulsiveForceVectorNode.setHeadSize(FORCE_ARROW_HEAD_WIDTH, FORCE_ARROW_HEAD_LENGTH);
    m_repulsiveForceVectorNode.setTailWidth(FORCE_ARROW_TAIL_WIDTH);
    m_repulsiveForceVectorNode.setVisible(false);
    m_totalForceVectorNode = new Vector2DNode(0, 0, TOTAL_FORCE_ARROW_REFERENCE_MAGNITUDE, TOTAL_FORCE_ARROW_REFERENCE_LENGTH);
    m_totalForceVectorNode.setMagnitudeAngle(0, 0);
    addChild(m_totalForceVectorNode);
    m_totalForceVectorNode.setArrowFillPaint(TOTAL_FORCE_COLOR);
    m_totalForceVectorNode.setHeadSize(FORCE_ARROW_HEAD_WIDTH, FORCE_ARROW_HEAD_LENGTH);
    m_totalForceVectorNode.setTailWidth(FORCE_ARROW_TAIL_WIDTH);
    m_totalForceVectorNode.setVisible(false);
}
  
  return inherit(ParticleNode,ParticleForceNode,{
//-----------------------------------------------------------------------------
// Accessor Methods
//-----------------------------------------------------------------------------
/**
     * Set the levels of attractive and repulsive forces being experienced by
     * the particles in the model so that they may be represented as force
     * vectors.
     */
setForces: function( attractiveForce,  repulsiveForce) {
    m_attractiveForce = attractiveForce;
    m_repulsiveForce = repulsiveForce;
    updateForceVectors();
},
//-----------------------------------------------------------------------------
// Other Public Methods
//-----------------------------------------------------------------------------
setShowAttractiveForces: function( showAttractiveForces) {
    m_attractiveForceVectorNode.setVisible(showAttractiveForces);
},
setShowRepulsiveForces: function( showRepulsiveForces) {
    m_repulsiveForceVectorNode.setVisible(showRepulsiveForces);
},
setShowTotalForces: function( showTotalForce) {
    m_totalForceVectorNode.setVisible(showTotalForce);
},
//-----------------------------------------------------------------------------
// Private Methods
//-----------------------------------------------------------------------------
/**
     * Update the force vectors to reflect the forces being experienced by the
     * atom.
     */
updateForceVectors: function() {
    m_attractiveForceVectorNode.setMagnitudeAngle(m_attractiveForce, 0);
    m_repulsiveForceVectorNode.setMagnitudeAngle(m_repulsiveForce, 0);
    m_totalForceVectorNode.setMagnitudeAngle(m_attractiveForce + m_repulsiveForce, 0);
}
},
//statics
{
ATTRACTIVE_FORCE_COLOR:ATTRACTIVE_FORCE_COLOR,
REPULSIVE_FORCE_COLOR:REPULSIVE_FORCE_COLOR,
TOTAL_FORCE_COLOR:TOTAL_FORCE_COLOR
});
} );

