// Copyright 2002-2011, University of Colorado
/**
 * This class is a Piccolo PNode extension that represents a particle in the view.
 *
 * @author John Blanco
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var BasicStroke = require('java.awt.BasicStroke');
var Color = require( 'SCENERY/util/Color' );
var Paint = require('java.awt.Paint');
var Stroke = require('java.awt.Stroke');
var Vector2 = require('java.awt.geom.Vector2');
var PhetColorScheme = require('edu.colorado.phet.common.phetcommon.view.PhetColorScheme');
var RoundGradientPaint = require('edu.colorado.phet.common.phetcommon.view.graphics.RoundGradientPaint');
var ColorUtils = require('edu.colorado.phet.common.phetcommon.view.util.ColorUtils');
var SphericalNode = require('edu.colorado.phet.common.piccolophet.nodes.SphericalNode');
var ArgonAtom = require('STATES_OF_MATTER/states-of-matter/model/particle/ArgonAtom');
var ConfigurableStatesOfMatterAtom = require('STATES_OF_MATTER/states-of-matter/model/particle/ConfigurableStatesOfMatterAtom');
var HydrogenAtom = require('STATES_OF_MATTER/states-of-matter/model/particle/HydrogenAtom');
var HydrogenAtom2 = require('STATES_OF_MATTER/states-of-matter/model/particle/HydrogenAtom2');
var NeonAtom = require('STATES_OF_MATTER/states-of-matter/model/particle/NeonAtom');
var OxygenAtom = require('STATES_OF_MATTER/states-of-matter/model/particle/OxygenAtom');
var StatesOfMatterAtom = require('STATES_OF_MATTER/states-of-matter/model/particle/StatesOfMatterAtom');
var Node = require('SCENERY/nodes/Node');
 
//----------------------------------------------------------------------------
// Class Data
//----------------------------------------------------------------------------
var OVERLAP_ENLARGEMENT_FACTOR = 1.25;
 
  //private
var STROKE_WIDTH = 20;
 
  //private
var STROKE = new BasicStroke(STROKE_WIDTH);
function ParticleNode( particle,  mvt) {
    //----------------------------------------------------------------------------
    // Instance Data
    //----------------------------------------------------------------------------
    this.m_particle;
     
  //private
this.m_mvt;
     
  //private
this.m_position;
     
  //private
this.m_sphere;
     
  //private
this.m_useGradient = false;
     
  //private
this.m_overlapEnabled = false;
    // If the user doesn't specify, a basic circle with no stroke and no gradient is assumed.
    this(particle, mvt, false, false, false);
}
/**
     * Main constructor.
     *
     * @param particle      - The particle in the model that this node will represent in the view.
     * @param mvt           - The model view transform for transforming particle position.
     * @param useGradient   - True to use a gradient when displaying the node, false if not.  The gradient is
     *                      computationally intensive to create, so use only when needed.
     * @param useStroke
     * @param enableOverlap - True if the node should be larger than the actual particle, thus allowing particles
     */
function ParticleNode( particle,  mvt,  useGradient,  useStroke,  enableOverlap) {
    //----------------------------------------------------------------------------
    // Instance Data
    //----------------------------------------------------------------------------
    this.m_particle;
     
  //private
this.m_mvt;
     
  //private
this.m_position;
     
  //private
this.m_sphere;
     
  //private
this.m_useGradient = false;
     
  //private
this.m_overlapEnabled = false;
    if ((mvt == null) || (particle == null)) {
        throw new IllegalArgumentException();
    }
    m_particle = particle;
    m_mvt = mvt;
    m_useGradient = useGradient;
    m_overlapEnabled = enableOverlap;
    // Local initialization.
    m_position = new Vector2();
    // Set ourself up to listen to this particle.
    particle.addListener(new StatesOfMatterAtom.Adapter().withAnonymousClassBody( {
        positionChanged: function() {
            updatePosition();
        },
        particleRemoved: function( particle) {
            handleParticleRemoved();
        },
        radiusChanged: function() {
            handleParticleRadiusChanged();
        }
    }));
    // Decide of the diameter of the sphere/circle.
    var sphereDiameter = particle.getRadius() * 2;
    if (m_overlapEnabled) {
        // inter-particle collisions.
        sphereDiameter = sphereDiameter * OVERLAP_ENLARGEMENT_FACTOR;
    }
    // will be less computationally intensive to move it around.
    m_sphere = new SphericalNode(sphereDiameter, choosePaint(particle), useGradient);
    addChild(m_sphere);
    setStrokeEnabled(useStroke);
    // Set ourself to be non-pickable so that we don't get mouse events.
    setPickable(false);
    setChildrenPickable(false);
    updatePosition();
}
  
  return inherit(Node,ParticleNode,{
//----------------------------------------------------------------------------
// Public Methods
//----------------------------------------------------------------------------
getGradientEnabled: function() {
    return m_useGradient;
},
setGradientEnabled: function( gradientEnabled) {
    if (m_useGradient != gradientEnabled) {
        m_useGradient = gradientEnabled;
        if (m_useGradient) {
            m_sphere.setConvertToImage(true);
            m_sphere.setPaint(choosePaint(m_particle));
        } else {
            m_sphere.setConvertToImage(false);
            m_sphere.setPaint(chooseColor(m_particle));
        }
    }
},
setStrokeEnabled: function( strokeEnabled) {
    if (strokeEnabled) {
        m_sphere.setStroke(STROKE);
        m_sphere.setStrokePaint(Color.BLACK);
    } else {
        m_sphere.setStroke(null);
        m_sphere.setStrokePaint(null);
    }
},
//----------------------------------------------------------------------------
// Private Methods
//----------------------------------------------------------------------------
 
  //private
updatePosition: function() {
    if (m_particle != null) {
        m_mvt.modelToView(m_particle.getPositionReference(), m_position);
        setOffset(m_position);
    }
},
/**
     * Handle the removal of the particle within the model that is being
     * represented in the view by this particle node.  This is done by
     * removing ourself from the canvas and by cleaning up any memory
     * references so that we can be garbage collected.
     */
 
  //private
handleParticleRemoved: function() {
    // Remove ourself from the canvas.
    var parent = getParent();
    if (parent != null) {
        parent.removeChild(this);
    }
    // Remove all children, since they have a reference to this object.
    removeAllChildren();
    // Explicitly clear our reference to the particle in the model.
    m_particle = null;
},
/**
     * If the radius of the particle changes, we need to redraw ourself to
     * correspond.
     */
handleParticleRadiusChanged: function() {
    if (m_useGradient) {
        // If the size changes, the gradient must also change to match.
        m_sphere.setPaint(choosePaint(m_particle));
    }
    var sphereDiameter = m_particle.getRadius() * 2;
    if (m_overlapEnabled) {
        // happen when the particles collide.
        sphereDiameter = sphereDiameter * OVERLAP_ENLARGEMENT_FACTOR;
    }
    m_sphere.setDiameter(sphereDiameter);
},
/**
     * Select the color for this particle.
     *
     * @return Color to use for this particle.
     */
choosePaint: function( atom) {
    var baseColor = chooseColor(atom);
    var darkenedBaseColor = ColorUtils.darkerColor(baseColor, 0.9);
    var transparentDarkenedBasedColor = new Color(darkenedBaseColor.getRed(), darkenedBaseColor.getGreen(), darkenedBaseColor.getBlue(), 20);
    if (m_useGradient) {
        var radius = m_overlapEnabled ? atom.getRadius() * OVERLAP_ENLARGEMENT_FACTOR : atom.getRadius();
        return (new RoundGradientPaint(0, 0, baseColor, new Vector2(-radius, radius), transparentDarkenedBasedColor));
    } else {
        return baseColor;
    }
},
/**
     * Choose the base color for the node based on the type of atom.
     *
     * @param atom
     * @return
     */
chooseColor: function( atom) {
    var baseColor;
    if (atom instanceof ArgonAtom) {
        baseColor = Color.pink;
    } else if (atom instanceof NeonAtom) {
        baseColor = new Color(112, 212, 255);
    } else if (atom instanceof OxygenAtom) {
        baseColor = PhetColorScheme.RED_COLORBLIND;
    } else if (atom instanceof HydrogenAtom) {
        baseColor = Color.WHITE;
    } else if (atom instanceof HydrogenAtom2) {
        baseColor = Color.WHITE;
    } else if (atom instanceof ConfigurableStatesOfMatterAtom) {
        baseColor = new Color(204, 102, 204);
    } else {
        baseColor = Color.PINK;
    }
    return baseColor;
}
},
//statics
{
OVERLAP_ENLARGEMENT_FACTOR:OVERLAP_ENLARGEMENT_FACTOR
});
} );

