// Copyright 2002-2011, University of Colorado
/**
 * This class represents a node that looks like a large finger, which can be
 * used to push down on things.
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var Color = require( 'SCENERY/util/Color' );
var Cursor = require('java.awt.Cursor');
var Vector2 = require('java.awt.geom.Vector2');
var Rectangle = require('KITE/Rectangle');
var CursorHandler = require('edu.colorado.phet.common.piccolophet.event.CursorHandler');
var ArrowNode = require('edu.colorado.phet.common.piccolophet.nodes.ArrowNode');
var StatesOfMatterConstants = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterConstants');
var StatesOfMatterResources = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterResources');
var MultipleParticleModel = require('STATES_OF_MATTER/states-of-matter/model/MultipleParticleModel');
var Node = require('SCENERY/nodes/Node');
var PBasicInputEventHandler = require('edu.umd.cs.piccolo.event.PBasicInputEventHandler');
var PDragEventHandler = require('edu.umd.cs.piccolo.event.PDragEventHandler');
var PInputEvent = require('edu.umd.cs.piccolo.event.PInputEvent');
var PImage = require('edu.umd.cs.piccolo.nodes.PImage');
var PDimension = require('edu.umd.cs.piccolo.util.PDimension');
 
//----------------------------------------------------------------------------
// Class Data
//----------------------------------------------------------------------------
// Width of the finger node as a proportion of the width of the particle
// container.
 
  //private
var HAND_NODE_WIDTH_PROPORTION = 0.55;
// Horizontal position of the node as function of the container width.
 
  //private
var NODE_X_POS_PROPORTION = 0.30;
// File name of the primary image.
var PRIMARY_IMAGE = "finger-4.png";
  
  // static class: MovementHintNode
var MovementHintNode = 
/**
     * This class represents a node that is meant to make it clear to the user
     * that the hand can be moved up and down.
     */
 
  //private
define( function( require ) {
 
  //private
var ARROW_COLOR = Color.GREEN;
 
  //private
var INVISIBLE_ARROW_COLOR = new Color(0, 0, 0, 0);
 
  //private
var ARROW_LENGTH = 1000;
 
  //private
var ARROW_HEAD_WIDTH = 1000;
 
  //private
var ARROW_HEAD_HEIGHT = 500;
 
  //private
var ARROW_TAIL_WIDTH = 500;
 
  //private
var DISTANCE_BETWEEN_ARROWS = 250;
 
  //private
function MovementHintNode( model) {
     
  //private
this.m_upArrowNode;
     
  //private
this.m_downArrow;
     
  //private
this.m_model;
    m_model = model;
    // Add the up arrow.
    m_upArrowNode = new ArrowNode(new Vector2(ARROW_HEAD_WIDTH / 2, ARROW_LENGTH), new Vector2(ARROW_HEAD_WIDTH / 2, 0), ARROW_HEAD_HEIGHT, ARROW_HEAD_WIDTH, ARROW_TAIL_WIDTH).withAnonymousClassBody( {
        initializer:function(){
            setPaint(ARROW_COLOR);
        }
    });
    addChild(m_upArrowNode);
    // Add the down arrow.
    m_downArrow = new ArrowNode(new Vector2(ARROW_HEAD_WIDTH / 2, ARROW_LENGTH + DISTANCE_BETWEEN_ARROWS), new Vector2(ARROW_HEAD_WIDTH / 2, ARROW_LENGTH * 2 + DISTANCE_BETWEEN_ARROWS), ARROW_HEAD_HEIGHT, ARROW_HEAD_WIDTH, ARROW_TAIL_WIDTH).withAnonymousClassBody( {
        initializer:function(){
            setPaint(ARROW_COLOR);
        }
    });
    addChild(m_downArrow);
    model.addListener(new MultipleParticleModel.Adapter().withAnonymousClassBody( {
        containerSizeChanged: function() {
            updateArrowVisibility();
        }
    }));
    // Make sure this node doesn't intercept mouse events.
    setPickable(false);
    setChildrenPickable(false);
}
  
  return inherit(Node,MovementHintNode,{
 
  //private
updateArrowVisibility: function() {
    if (m_model.getParticleContainerHeight() == StatesOfMatterConstants.PARTICLE_CONTAINER_INITIAL_HEIGHT) {
        // At the height limit, so only show the down arrow.
        m_upArrowNode.setPaint(ARROW_COLOR);
        m_upArrowNode.setVisible(false);
    } else if (m_model.getParticleContainerHeight() == 0) {
        // Particle container all the way down, so show only the up arrow.
        m_upArrowNode.setPaint(INVISIBLE_ARROW_COLOR);
        m_upArrowNode.setVisible(false);
    } else {
        m_upArrowNode.setVisible(true);
        m_upArrowNode.setVisible(true);
    }
}
});
} );;
function PointingHandNode( model) {
    //----------------------------------------------------------------------------
    // Instance Data
    //----------------------------------------------------------------------------
     
  //private
this.m_fingerImageNode;
     
  //private
this.m_model;
     
  //private
this.m_mouseMovementAmount;
     
  //private
this.m_containerSizeAtDragStart;
     
  //private
this.m_mouseOver = false;
     
  //private
this.m_beingDragged = false;
     
  //private
this.m_hintNode;
    m_model = model;
    var containerRect = m_model.getParticleContainerRect();
    var desiredHandWidth = containerRect.getWidth() * HAND_NODE_WIDTH_PROPORTION;
    // Listen to the model for notifications of changes to the container size.
    m_model.addListener(new MultipleParticleModel.Adapter().withAnonymousClassBody( {
        containerSizeChanged: function() {
            handleContainerSizeChanged();
        }
    }));
    // Load and scale the image.
    m_fingerImageNode = StatesOfMatterResources.getImageNode(PRIMARY_IMAGE);
    var scale = desiredHandWidth / m_fingerImageNode.getFullBoundsReference().width;
    m_fingerImageNode.scale(scale);
    // that the node can be moved.
    m_fingerImageNode.setPickable(true);
    m_fingerImageNode.addInputEventListener(new CursorHandler(Cursor.N_RESIZE_CURSOR));
    // placed the mouse over the finger.
    m_hintNode = new MovementHintNode(m_model).withAnonymousClassBody( {
        initializer:function(){
            // coded values were empirically determined.
            setOffset(m_fingerImageNode.getFullBoundsReference().getMaxX() + 150, m_fingerImageNode.getFullBoundsReference().getMaxY() - getFullBoundsReference().getHeight() - 1000);
            setVisible(false);
        }
    });
    addChild(m_hintNode);
    // Create a listener that tracks whether the user's mouse is over this node.
    m_fingerImageNode.addInputEventListener(new PBasicInputEventHandler().withAnonymousClassBody( {
        mouseEntered: function( event) {
            m_mouseOver = true;
            updateHintVisibility();
        },
        mouseExited: function( event) {
            m_mouseOver = false;
            updateHintVisibility();
        }
    }));
    // Set ourself up to listen for and handle mouse dragging events.
    m_fingerImageNode.addInputEventListener(new PDragEventHandler().withAnonymousClassBody( {
        startDrag: function( event) {
            super.startDrag(event);
            handleMouseStartDragEvent();
        },
        drag: function( event) {
            handleMouseDragEvent(event);
        },
        endDrag: function( event) {
            super.endDrag(event);
            handleMouseEndDragEvent();
        }
    }));
    // Add the finger node as a child.
    addChild(m_fingerImageNode);
    // Set our initial offset.
    setOffset(containerRect.getX() + containerRect.getWidth() * NODE_X_POS_PROPORTION, -getFullBoundsReference().height);
}
  
  return inherit(Node,PointingHandNode,{
//----------------------------------------------------------------------------
// Private Methods
//----------------------------------------------------------------------------
 
  //private
handleMouseDragEvent: function( event) {
    var draggedNode = event.getPickedNode();
    var d = event.getDeltaRelativeTo(draggedNode);
    draggedNode.localToParent(d);
    m_mouseMovementAmount += d.getHeight();
    // Resize the container based on the amount that the node has moved.
    m_model.setTargetParticleContainerHeight(m_containerSizeAtDragStart - m_mouseMovementAmount);
},
 
  //private
handleMouseStartDragEvent: function() {
    m_beingDragged = true;
    m_mouseMovementAmount = 0;
    m_containerSizeAtDragStart = m_model.getParticleContainerHeight();
    updateHintVisibility();
},
 
  //private
handleMouseEndDragEvent: function() {
    // in size that is currently underway.
    m_model.setTargetParticleContainerHeight(m_model.getParticleContainerHeight());
    m_beingDragged = false;
    updateHintVisibility();
},
 
  //private
handleContainerSizeChanged: function() {
    var containerRect = m_model.getParticleContainerRect();
    if (!m_model.getContainerExploded()) {
        setOffset(getFullBoundsReference().x, StatesOfMatterConstants.PARTICLE_CONTAINER_INITIAL_HEIGHT - containerRect.getHeight() - getFullBoundsReference().height);
    } else {
        // quickly.
        setOffset(getFullBoundsReference().x, StatesOfMatterConstants.PARTICLE_CONTAINER_INITIAL_HEIGHT - (containerRect.getHeight() * 2) - getFullBoundsReference().height);
    }
},
 
  //private
updateHintVisibility: function() {
    m_hintNode.setVisible(m_mouseOver || m_beingDragged);
},
},
//statics
{
PRIMARY_IMAGE:PRIMARY_IMAGE
});
} );

