// Copyright 2002-2011, University of Colorado
/**
 * This class is the graphical representation of a stove that can be used to
 * heat or cool things.
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var BasicStroke = require('java.awt.BasicStroke');
var Color = require( 'SCENERY/util/Color' );
var GradientPaint = require('java.awt.GradientPaint');
var Paint = require('java.awt.Paint');
var Shape = require('java.awt.Shape');
var KeyEvent = require('java.awt.event.KeyEvent');
var Ellipse2D = require('java.awt.geom.Ellipse2D');
var Rectangle = require('KITE/Rectangle');
var SwingUtilities = require('javax.swing.SwingUtilities');
var Property = require('AXON/Property');
var DoubleRange = require('edu.colorado.phet.common.phetcommon.util.DoubleRange');
var VoidFunction1 = require('edu.colorado.phet.common.phetcommon.util.function.VoidFunction1');
var ColorUtils = require('edu.colorado.phet.common.phetcommon.view.util.ColorUtils');
var DoubleGeneralPath = require('edu.colorado.phet.common.phetcommon.view.util.DoubleGeneralPath');
var PhetPPath = require('edu.colorado.phet.common.piccolophet.nodes.PhetPPath');
var PiccoloTestFrame = require('edu.colorado.phet.common.piccolophet.test.PiccoloTestFrame');
var StatesOfMatterResources = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterResources');
var MultipleParticleModel = require('STATES_OF_MATTER/states-of-matter/model/MultipleParticleModel');
var Node = require('SCENERY/nodes/Node');
var PBasicInputEventHandler = require('edu.umd.cs.piccolo.event.PBasicInputEventHandler');
var PInputEvent = require('edu.umd.cs.piccolo.event.PInputEvent');
var PImage = require('edu.umd.cs.piccolo.nodes.PImage');
 
//-------------------------------------------------------------------------
// Class Data
//-------------------------------------------------------------------------
// Width of the burner output - much of the rest of the size of the stove
// derives from this value.
// In screen coords, which are close to pixels.
 
  //private
var WIDTH = 250;
// In screen coords, which are close to pixels.
 
  //private
var HEIGHT = WIDTH * 0.75;
// In screen coords, which are close to pixels.
 
  //private
var BURNER_OPENING_HEIGHT = WIDTH * 0.1;
// Basic color used for the stove.
 
  //private
var BASE_COLOR = new Color(159, 182, 205);
// Valid range of heat values.
 
  //private
var heatRange = new DoubleRange(-1, 1);
function StoveNode( model) {
    //-------------------------------------------------------------------------
    // Instance Data
    //-------------------------------------------------------------------------
     
  //private
this.m_fireImage;
     
  //private
this.m_iceImage;
     
  //private
this.m_model;
    // Heat value, ranges from -1 to +1.
     
  //private
this.m_heat = new Property(0.0);
     
  //private
this.m_burner;
    m_model = model;
    // Register for events from the model that will affect the stove.
    if (m_model != null) {
        m_model.addListener(new MultipleParticleModel.Adapter().withAnonymousClassBody( {
            resetOccurred: function() {
                m_heat.set(0.0);
            },
            moleculeTypeChanged: function() {
                m_heat.set(0.0);
            },
            containerExplodedStateChanged: function( containerExploded) {
                m_heat.set(0.0);
            }
        }));
    }
    // Create the body of the stove.
    var stoveBodyShape = new DoubleGeneralPath().withAnonymousClassBody( {
        initializer:function(){
            var bottomWidth = WIDTH * 0.8;
            // Start in upper left corner.
            moveTo(0, 0);
            // Curve to upper right corner.
            curveTo(0, BURNER_OPENING_HEIGHT / 2, WIDTH, BURNER_OPENING_HEIGHT / 2, WIDTH, 0);
            // Line to lower right corner.
            lineTo((WIDTH + bottomWidth) / 2, HEIGHT);
            curveTo((WIDTH + bottomWidth) / 2, HEIGHT + BURNER_OPENING_HEIGHT / 2, (WIDTH - bottomWidth) / 2, HEIGHT + BURNER_OPENING_HEIGHT / 2, (WIDTH - bottomWidth) / 2, // Curve to lower left corner.
            HEIGHT);
            // Line back to the upper left corner.
            lineTo(0, 0);
            // Just to be sure.
            closePath();
        }
    });
    var stoveBodyPaint = new GradientPaint(0, 0, ColorUtils.brighterColor(BASE_COLOR, 0.5), WIDTH, 0, ColorUtils.darkerColor(BASE_COLOR, 0.5));
    m_burner = new PhetPPath(stoveBodyShape.getGeneralPath(), stoveBodyPaint, new BasicStroke(1), Color.BLACK);
    // Create the inside bowl of the burner, which is an ellipse.
    var burnerInteriorShape = new Ellipse2D.Number(0, 0, WIDTH, BURNER_OPENING_HEIGHT);
    var burnerInteriorPaint = new GradientPaint(0, 0, ColorUtils.darkerColor(BASE_COLOR, 0.25), WIDTH, 0, ColorUtils.brighterColor(BASE_COLOR, 0.5));
    var burnerInterior = new PhetPPath(burnerInteriorShape, burnerInteriorPaint, new BasicStroke(1), Color.LIGHT_GRAY);
    // Create the slider.
    var stoveControlSlider = new StoveControlSliderNode(m_heat).withAnonymousClassBody( {
        initializer:function(){
            // may be scaled differently for different translations.
            var maxWidth = WIDTH * 0.8;
            var maxHeight = HEIGHT * 0.8;
            var scale = Math.min(maxWidth / getFullBoundsReference().width, maxHeight / getFullBoundsReference().height);
            setScale(scale);
        }
    });
    // Add the images for fire and ice that come out of the stove.
    m_fireImage = StatesOfMatterResources.getImageNode("flame.png");
    m_fireImage.setScale((WIDTH * 0.8) / m_fireImage.getFullBoundsReference().getWidth());
    m_iceImage = StatesOfMatterResources.getImageNode("ice-cube-stack.png");
    m_iceImage.setScale((WIDTH * 0.8) / m_iceImage.getFullBoundsReference().getWidth());
    // desired layering.
    addChild(burnerInterior);
    addChild(m_fireImage);
    addChild(m_iceImage);
    addChild(m_burner);
    addChild(stoveControlSlider);
    // Note - Goes a little negative in Y direction.
    burnerInterior.setOffset(0, -burnerInterior.getFullBoundsReference().height / 2);
    stoveControlSlider.setOffset(WIDTH / 2 - stoveControlSlider.getFullBoundsReference().width / 2, HEIGHT / 2 - stoveControlSlider.getFullBoundsReference().height / 2 + burnerInterior.getFullBoundsReference().height / 2);
    // accordingly.
    m_heat.addObserver(new VoidFunction1().withAnonymousClassBody( {
        apply: function( heat) {
            update();
            if (m_model != null) {
                m_model.setHeatingCoolingAmount(heat);
            }
        }
    }));
    // add and remove heat.
    addInputEventListener(new PBasicInputEventHandler().withAnonymousClassBody( {
        mousePressed: function( event) {
            // Get the keyboard focus.
            event.getInputManager().setKeyboardFocus(event.getPath());
        },
        keyPressed: function( event) {
            var numIncrements = 50;
            switch(event.getKeyCode()) {
                case KeyEvent.VK_UP:
                case KeyEvent.VK_KP_UP:
                    m_heat.set(Math.min(m_heat.get() + heatRange.getLength() / numIncrements, 1));
                    break;
                case KeyEvent.VK_DOWN:
                case KeyEvent.VK_KP_DOWN:
                    m_heat.set(Math.max(m_heat.get() - heatRange.getLength() / numIncrements, -1));
                    break;
                case KeyEvent.VK_ESCAPE:
                case KeyEvent.VK_0:
                case KeyEvent.VK_NUMPAD0:
                case KeyEvent.VK_ENTER:
                    m_heat.set(0.0);
                    break;
                default:
                    // Ignore the key.
                    break;
            }
        }
    }));
    update();
}
  
  return inherit(Node,StoveNode,{
//-------------------------------------------------------------------------
// Methods
//-------------------------------------------------------------------------
 
  //private
update: function() {
    if (m_heat.get() > 0) {
        m_fireImage.setOffset(m_burner.getFullBoundsReference().width / 2 - m_fireImage.getFullBoundsReference().width / 2, -m_heat.get() * m_fireImage.getFullBoundsReference().height * 0.9);
        m_iceImage.setOffset(m_burner.getFullBoundsReference().width / 2 - m_iceImage.getFullBoundsReference().width / 2, 0);
    } else if (m_heat.get() <= 0) {
        m_iceImage.setOffset(m_burner.getFullBoundsReference().width / 2 - m_iceImage.getFullBoundsReference().width / 2, m_heat.get() * m_iceImage.getFullBoundsReference().height * 0.9);
        m_fireImage.setOffset(m_burner.getFullBoundsReference().width / 2 - m_fireImage.getFullBoundsReference().width / 2, 0);
    }
    m_iceImage.setVisible(m_heat.get() < 0);
    m_fireImage.setVisible(m_heat.get() > 0);
},
main: function( args) {
    SwingUtilities.invokeLater(new Runnable().withAnonymousClassBody( {
        run: function() {
            new PiccoloTestFrame("Stove Node Test").withAnonymousClassBody( {
                initializer:function(){
                    addNode(new PhetPPath(new Rectangle.Number(0, 0, 1000, 1000), Color.black));
                    addNode(new StoveNode(null).withAnonymousClassBody( {
                        initializer:function(){
                            setOffset(100, 200);
                        }
                    }));
                    setBackground(Color.black);
                }
            }).setVisible(true);
        }
    }));
}
});
} );

