// Copyright 2002-2011, University of Colorado
/**
 * This class defines a PNode that has a liquid thermometer and a numerical
 * readout that can display the temperature in degrees Kelvin or degrees
 * Celsius.
 *
 * @author John Blanco
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var Color = require( 'SCENERY/util/Color' );
var RoundRectangle2D = require('java.awt.geom.RoundRectangle2D');
var DecimalFormat = require('java.text.DecimalFormat');
var Property = require('AXON/Property');
var VoidFunction1 = require('edu.colorado.phet.common.phetcommon.util.function.VoidFunction1');
var PhetFont = require('edu.colorado.phet.common.phetcommon.view.util.PhetFont');
var LiquidExpansionThermometerNode = require('edu.colorado.phet.common.piccolophet.nodes.LiquidExpansionThermometerNode');
var StatesOfMatterStrings = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterStrings');
var TemperatureUnits = require('STATES_OF_MATTER/states-of-matter/view/TemperatureUnits');
var Node = require('SCENERY/nodes/Node');
var PPath = require('edu.umd.cs.piccolo.nodes.PPath');
var PText = require('edu.umd.cs.piccolo.nodes.PText');
var PDimension = require('edu.umd.cs.piccolo.util.PDimension');
 
 
  //private
var THERMOMETER_WIDTH_PROPORTION = 0.38;
 
  //private
var LIQUID_THERMOMETER_SCALE_FACTOR = 20;
/**
     * Constructor.
     *
     * @param width           - Width on the canvas of this node.
     * @param height          - Height on the canvas of this node.
     * @param maxTempInKelvin - The maximum temperature in Kelvin that the thermometer can display.
     */
function CompositeThermometerNode( width,  height,  maxTempInKelvin,  temperatureUnitsProperty) {
     
  //private
this.m_liquidThermometer;
     
  //private
this.m_maxTemp;
     
  //private
this.m_digitalReadout;
      
  // non-static inner class: DigitalReadoutNode
    var DigitalReadoutNode = 
    /**
     * This class represents a node that will numerically display a value and
     * units.
     */
     
  //private
define( function( require ) {
     
  //private
var WIDTH_TO_HEIGHT_RATIO = 2.2;
     
  //private
var INSET_WIDTH_RATIO = 0.95;
    /**
         * Constructor.
         *
         * @param width
         * @param temperatureUnits
         */
    function DigitalReadoutNode( width,  temperatureUnits) {
         
  //private
this.BACKGROUND_COLOR = Color.YELLOW;
         
  //private
this.FOREGROUND_COLOR = Color.WHITE;
         
  //private
this.numberFormatter = new DecimalFormat("##0");
         
  //private
this.m_temperatureUnits = TemperatureUnits.KELVIN;
         
  //private
this.m_valueInKelvin = 0;
         
  //private
this.m_text;
         
  //private
this.m_units;
         
  //private
this.m_foregroundNode;
         
  //private
this.m_defaultTextScale = 1;
        var height = width / WIDTH_TO_HEIGHT_RATIO;
        var backgroundNode = new PPath(new RoundRectangle2D.Number(0, 0, width, height, height / 5, height / 5));
        backgroundNode.setPaint(BACKGROUND_COLOR);
        addChild(backgroundNode);
        var borderWidth = width * (1 - INSET_WIDTH_RATIO);
        m_foregroundNode = new PPath(new RoundRectangle2D.Number(0, 0, width - (borderWidth * 2), height - (borderWidth * 2), height / 5, height / 5));
        m_foregroundNode.setPaint(FOREGROUND_COLOR);
        addChild(m_foregroundNode);
        m_foregroundNode.setOffset(borderWidth, borderWidth);
        m_text = new PText("0");
        m_text.setFont(new PhetFont(12, true));
        m_defaultTextScale = m_foregroundNode.getFullBoundsReference().height * 0.8 / m_text.getFullBoundsReference().height;
        addChild(m_text);
        setTemperatureUnits(temperatureUnits);
        update();
    }
      
  return inherit(Node,DigitalReadoutNode,{
    /**
         * Set the units in which the temperature should be displayed.
         *
         * @param temperatureUnits
         */
    setTemperatureUnits: function( temperatureUnits) {
        // Doesn't handle all units.  Modify if needed.
        assert && assert(temperatureUnits == TemperatureUnits.KELVIN || temperatureUnits == TemperatureUnits.CELSIUS);
        m_temperatureUnits = temperatureUnits;
        // Update the string used for the units.
        switch(m_temperatureUnits) {
            case KELVIN:
                m_units = StatesOfMatterStrings.UNITS_K;
                break;
            case CELSIUS:
                m_units = StatesOfMatterStrings.UNITS_C;
                break;
            default:
                // Don't have support for specified units.
                assert && assert(false);
                m_units = "X";
                break;
        }
        update();
    },
    setValueKelvin: function( value) {
        m_valueInKelvin = value;
        update();
    },
     
  //private
update: function() {
        var valueString;
        if (m_temperatureUnits == TemperatureUnits.CELSIUS) {
            valueString = numberFormatter.format(Math.round(m_valueInKelvin - 273.15));
        } else {
            valueString = numberFormatter.format(Math.round(m_valueInKelvin));
        }
        if (m_units != null) {
            valueString += " ";
            valueString += m_units;
        }
        m_text.setScale(m_defaultTextScale);
        m_text.setText(valueString);
        if (m_text.getFullBoundsReference().width > m_foregroundNode.getFullBoundsReference().width * 0.95) {
            // Scale the text to fit in the readout.
            m_text.setScale(1);
            m_text.setScale(m_foregroundNode.getFullBoundsReference().width * 0.95 / m_text.getFullBoundsReference().width);
        }
        m_text.setOffset(m_foregroundNode.getFullBoundsReference().getCenterX() - m_text.getFullBoundsReference().width / 2, m_foregroundNode.getFullBoundsReference().getCenterY() - m_text.getFullBoundsReference().height / 2);
    }
    });
} );m_maxTemp = maxTempInKelvin;
    // Add the digital readout.
    m_digitalReadout = new DigitalReadoutNode(width, temperatureUnitsProperty.get());
    addChild(m_digitalReadout);
    // sure why.  It has something to do with clipping regions.  See Unfuddle ticket #656.
    m_liquidThermometer = new LiquidExpansionThermometerNode(new PDimension(width * THERMOMETER_WIDTH_PROPORTION / LIQUID_THERMOMETER_SCALE_FACTOR, height / LIQUID_THERMOMETER_SCALE_FACTOR));
    m_liquidThermometer.setTicks(height / 10 / LIQUID_THERMOMETER_SCALE_FACTOR, Color.BLACK, height / 3500);
    m_liquidThermometer.scale(LIQUID_THERMOMETER_SCALE_FACTOR);
    m_liquidThermometer.setOffset(0, m_digitalReadout.getFullBoundsReference().height * 1.1);
    addChild(m_liquidThermometer);
    // be displayed in Kelvin or Celsius.
    temperatureUnitsProperty.addObserver(new VoidFunction1().withAnonymousClassBody( {
        apply: function( temperatureUnits) {
            m_digitalReadout.setTemperatureUnits(temperatureUnits);
        }
    }));
}
  
  return inherit(Node,CompositeThermometerNode,{
setTemperatureInDegreesKelvin: function( degrees) {
    m_digitalReadout.setValueKelvin(degrees);
    m_liquidThermometer.setLiquidHeight(Math.min(degrees / m_maxTemp, 1.0));
},
});
} );

