// Copyright 2002-2011, University of Colorado
/**
 * This class represents a node that displays a dial gauge, which is a
 * circular instrument that can be used to portray measurements of temperature,
 * pressure, etc.
 *
 * @author John Blanco
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var BasicStroke = require('java.awt.BasicStroke');
var Color = require( 'SCENERY/util/Color' );
var GradientPaint = require('java.awt.GradientPaint');
var Paint = require('java.awt.Paint');
var Shape = require('java.awt.Shape');
var Stroke = require('java.awt.Stroke');
var Ellipse2D = require('java.awt.geom.Ellipse2D');
var GeneralPath = require('java.awt.geom.GeneralPath');
var Line2D = require('java.awt.geom.Line2D');
var Rectangle = require('KITE/Rectangle');
var RoundRectangle2D = require('java.awt.geom.RoundRectangle2D');
var DecimalFormat = require('java.text.DecimalFormat');
var SwingUtilities = require('javax.swing.SwingUtilities');
var PhetFont = require('edu.colorado.phet.common.phetcommon.view.util.PhetFont');
var PhetPPath = require('edu.colorado.phet.common.piccolophet.nodes.PhetPPath');
var PiccoloTestFrame = require('edu.colorado.phet.common.piccolophet.test.PiccoloTestFrame');
var StatesOfMatterStrings = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterStrings');
var Node = require('SCENERY/nodes/Node');
var PPath = require('edu.umd.cs.piccolo.nodes.PPath');
var PText = require('edu.umd.cs.piccolo.nodes.PText');
 
//------------------------------------------------------------------------
// Class Data
//------------------------------------------------------------------------
// Constants the control the appearance of the dial gauge.
 
  //private
var BACKGROUND_COLOR = new Color(245, 255, 250);
 
  //private
var BORDER_COLOR = Color.LIGHT_GRAY;
 
  //private
var NEEDLE_COLOR = Color.RED;
// Size of border wrt overall diameter.
 
  //private
var BORDER_SCALE_FACTOR = 0.015;
// Length of tick marks wrt overall diameter. 
 
  //private
var TICK_MARK_LENGTH_SCALE_FACTOR = 0.03;
// Width of tick marks wrt overall diameter. 
 
  //private
var TICK_MARK_WIDTH_SCALE_FACTOR = 0.008;
// Length of needle wrt overall diameter. 
 
  //private
var NEEDLE_LENGTH_SCALE_FACTOR = 0.55;
// Width of needle wrt overall diameter. 
 
  //private
var NEEDLE_WIDTH_SCALE_FACTOR = 0.015;
// Diameter of attachment pin wrt overall diameter. 
 
  //private
var PIN_DIAMETER_SCALE_FACTOR = 0.020;
// Height of textual readout box wrt overall diameter. 
 
  //private
var TEXTUAL_READOUT_HEIGHT_SCALE_FACTOR = 0.1;
// Width of textual readout box wrt overall diameter. 
 
  //private
var TEXTUAL_READOUT_WIDTH_SCALE_FACTOR = 0.6;
// Stroke width textual readout box wrt overall diameter. 
 
  //private
var TEXTUAL_READOUT_STROKE_SCALE_FACTOR = 0.010;
 
  //private
var NUM_TICKMARKS = 19;
// In radians.
 
  //private
var GAUGE_START_ANGLE = -Math.PI * 1.15;
// In radians.
 
  //private
var GAUGE_END_ANGLE = Math.PI * 0.15;
 
  //private
var GAUGE_ANGLE_RANGE = GAUGE_END_ANGLE - GAUGE_START_ANGLE;
// Proportion of needle used as pointer.
 
  //private
var NEEDLE_SHIFT_PROPORTION = 0.75;
// Length of non-elbowed connector wrt overall diameter.
 
  //private
var CONNECTOR_LENGTH_PROPORATION = 0.30;
// Width of connector wrt overall diameter.
 
  //private
var CONNECTOR_WIDTH_PROPORATION = 0.15;
 
  //private
var NUMBER_FORMATTER = new DecimalFormat("##0.00");
function DialGaugeNode( diameter,  title,  minValue,  maxValue,  unitsLabel) {
    //------------------------------------------------------------------------
    // Instance Data
    //------------------------------------------------------------------------
     
  //private
this.m_diameter;
     
  //private
this.m_needle;
     
  //private
this.m_needleAngle;
     
  //private
this.m_connector;
     
  //private
this.m_needleLength;
     
  //private
this.m_textualReadout;
     
  //private
this.m_minValue;
     
  //private
this.m_maxValue;
     
  //private
this.m_unitsLabel;
     
  //private
this.m_textualReadoutBoxShape;
     
  //private
this.m_connectorPath;
     
  //private
this.m_elbowEnabled;
     
  //private
this.m_elbowHeight;
     
  //private
this.m_dialComponentsNode;
    m_minValue = minValue;
    m_maxValue = maxValue;
    m_unitsLabel = unitsLabel;
    m_elbowEnabled = false;
    m_diameter = diameter;
    // diameter.
    var borderWidth = diameter * BORDER_SCALE_FACTOR;
    m_dialComponentsNode = new Node();
    m_dialComponentsNode.setOffset(0, 0);
    // Create the node that will represent the face of the dial.
    var dialFace = new PPath(new Ellipse2D.Number(0, 0, diameter, diameter));
    dialFace.setPaint(BACKGROUND_COLOR);
    dialFace.setStrokePaint(BORDER_COLOR);
    dialFace.setStroke(new BasicStroke(borderWidth));
    m_dialComponentsNode.addChild(dialFace);
    // Add the tick marks.
    var tickSpace = (Math.PI * 6 / 4) / (NUM_TICKMARKS);
    var tickMarkStroke = new BasicStroke((diameter * TICK_MARK_WIDTH_SCALE_FACTOR));
    for (var theta = GAUGE_START_ANGLE; theta <= GAUGE_END_ANGLE; theta += tickSpace) {
        var tickMark = new PPath(new Line2D.Number(0, 0, diameter * TICK_MARK_LENGTH_SCALE_FACTOR, 0));
        tickMark.setStroke(tickMarkStroke);
        tickMark.rotate(theta);
        tickMark.setOffset((diameter / 2) + (diameter * 0.44) * Math.cos(theta), (diameter / 2) + (diameter * 0.44) * Math.sin(theta));
        m_dialComponentsNode.addChild(tickMark);
    }
    // Add the title.
    var gaugeTitle = new PText();
    gaugeTitle.setText(title);
    gaugeTitle.setFont(new PhetFont(12));
    gaugeTitle.scale((m_dialComponentsNode.getFullBoundsReference().width * 0.6) / gaugeTitle.getFullBoundsReference().width);
    gaugeTitle.setOffset(diameter / 2 - gaugeTitle.getFullBoundsReference().width / 2, diameter / 4);
    m_dialComponentsNode.addChild(gaugeTitle);
    // Add the textual readout display.
    m_textualReadoutBoxShape = new RoundRectangle2D.Number(0, 0, TEXTUAL_READOUT_WIDTH_SCALE_FACTOR * diameter, TEXTUAL_READOUT_HEIGHT_SCALE_FACTOR * diameter, 4, 4);
    var textualReadoutHighlight = new PPath(m_textualReadoutBoxShape);
    var highlightStrokeWidth = (diameter * TEXTUAL_READOUT_STROKE_SCALE_FACTOR * 3);
    textualReadoutHighlight.setStroke(new BasicStroke(highlightStrokeWidth));
    textualReadoutHighlight.setStrokePaint(Color.YELLOW);
    textualReadoutHighlight.setPaint(Color.WHITE);
    textualReadoutHighlight.setOffset(diameter / 2 - textualReadoutHighlight.getWidth() / 2 + highlightStrokeWidth / 2, diameter * 0.70);
    m_dialComponentsNode.addChild(textualReadoutHighlight);
    var textualReadoutBox = new PPath(m_textualReadoutBoxShape);
    var textBoxStrokeWidth = (diameter * TEXTUAL_READOUT_STROKE_SCALE_FACTOR);
    textualReadoutBox.setStroke(new BasicStroke(textBoxStrokeWidth));
    textualReadoutBox.setStrokePaint(Color.DARK_GRAY);
    textualReadoutBox.setOffset(diameter / 2 - textualReadoutBox.getWidth() / 2 + textBoxStrokeWidth / 2, diameter * 0.70);
    m_dialComponentsNode.addChild(textualReadoutBox);
    m_textualReadout = new PText(" ");
    m_textualReadout.setFont(new PhetFont(12));
    m_textualReadout.scale(textualReadoutBox.getHeight() * 0.8 / m_textualReadout.getFullBoundsReference().height);
    textualReadoutBox.addChild(m_textualReadout);
    // Add the needle.
    m_needleLength = diameter * NEEDLE_LENGTH_SCALE_FACTOR;
    m_needle = new PPath(new Line2D.Number(0, 0, m_needleLength, 0));
    m_needle.setStroke(new BasicStroke((diameter * NEEDLE_WIDTH_SCALE_FACTOR)));
    m_needle.setStrokePaint(NEEDLE_COLOR);
    m_needle.setOffset((diameter / 2) - (m_needleLength * (1 - NEEDLE_SHIFT_PROPORTION)), diameter / 2);
    m_needleAngle = GAUGE_START_ANGLE;
    m_needle.rotateAboutPoint(m_needleAngle, (m_needleLength * (1 - NEEDLE_SHIFT_PROPORTION)), 0);
    m_dialComponentsNode.addChild(m_needle);
    // Add a little pin in the center where the needle attaches to the face.
    var pinDiameter = PIN_DIAMETER_SCALE_FACTOR * diameter;
    var pin = new PPath(new Ellipse2D.Number(0, 0, pinDiameter, pinDiameter));
    pin.setPaint(Color.BLACK);
    pin.setOffset(diameter / 2 - pinDiameter / 2, diameter / 2 - pinDiameter / 2);
    m_dialComponentsNode.addChild(pin);
    // to use a gradient without crashing any Macs.
    m_connector = new PhetPPath();
    m_connector.setPaint(Color.BLUE);
    m_connectorPath = new GeneralPath();
    updateConnector();
    addChild(m_connector);
    // Now add the dial as a child of the main node.
    addChild(m_dialComponentsNode);
    // Set the initial value.
    setValue(m_minValue);
}
  
  return inherit(Node,DialGaugeNode,{
//------------------------------------------------------------------------
// Accessor Methods
//------------------------------------------------------------------------
//------------------------------------------------------------------------
// Other Public Methods
//------------------------------------------------------------------------
/**
     * Set the value of the pressure gauge.  The value must be within the
     * bounds set when the gauge was created.
     */
setValue: function( value) {
    var overload = false;
    // Limit the value.
    if (value < m_minValue) {
        value = m_minValue;
    } else if (value > m_maxValue) {
        value = m_maxValue;
        overload = true;
    }
    // Set the needle position.
    var normalizedValue = value / (m_maxValue - m_minValue);
    var targetNeedleAngle = GAUGE_START_ANGLE + (GAUGE_ANGLE_RANGE * normalizedValue);
    m_needle.rotateAboutPoint(targetNeedleAngle - m_needleAngle, m_needleLength * (1 - NEEDLE_SHIFT_PROPORTION), 0);
    m_needleAngle = targetNeedleAngle;
    // Set the textual readout.
    if (!overload) {
        m_textualReadout.setText(NUMBER_FORMATTER.format(value) + " " + m_unitsLabel);
        m_textualReadout.setTextPaint(Color.BLACK);
        m_textualReadout.setOffset(m_textualReadoutBoxShape.getWidth() / 2 - m_textualReadout.getFullBoundsReference().width / 2, m_textualReadoutBoxShape.getHeight() / 2 - m_textualReadout.getFullBoundsReference().height / 2);
    } else {
        m_textualReadout.setText(StatesOfMatterStrings.PRESSURE_GAUGE_OVERLOAD);
        m_textualReadout.setTextPaint(Color.RED);
        m_textualReadout.setOffset(m_textualReadoutBoxShape.getWidth() / 2 - m_textualReadout.getFullBoundsReference().width / 2, m_textualReadoutBoxShape.getHeight() / 2 - m_textualReadout.getFullBoundsReference().height / 2);
    }
},
/**
     * This turns on/off the "elbow" portion of the connector, which allows
     * the pressure gauge to connect to something above or below it.
     *
     * @param elbowEnabled
     */
setElbowEnabled: function( elbowEnabled) {
    m_elbowEnabled = elbowEnabled;
    // Create a different gradient if the elbow is enabled.
    var width = (CONNECTOR_WIDTH_PROPORATION * m_diameter);
    var length = (CONNECTOR_LENGTH_PROPORATION * m_diameter);
    var m_connectorPaint;
    if (m_elbowEnabled) {
        m_connectorPaint = new GradientPaint(length, width / 2, Color.BLUE, length + width, width / 2, Color.LIGHT_GRAY);
    } else {
        m_connectorPaint = new GradientPaint((m_diameter / 2), 0, Color.LIGHT_GRAY, (m_diameter / 2), (CONNECTOR_WIDTH_PROPORATION * m_diameter), Color.BLUE);
    }
    m_connector.setPaint(m_connectorPaint);
    updateConnector();
},
/**
     * Set the height of the elbow.  Height is specified with respect to the
     * vertical center of the node.
     */
setElbowHeight: function( height) {
    m_elbowHeight = height;
    updateConnector();
},
/**
     * Test harness.
     *
     * @param args
     */
main: function( args) {
    SwingUtilities.invokeLater(new Runnable().withAnonymousClassBody( {
        run: function() {
            var testFrame = new PiccoloTestFrame("Dial Gauge Test");
            var dialGaugeNode1 = new DialGaugeNode(200, "Pressure", 0, 1, "Atm");
            dialGaugeNode1.setOffset(50, 20);
            dialGaugeNode1.setValue(0.75);
            testFrame.addNode(dialGaugeNode1);
            var dialGaugeNode2 = new DialGaugeNode(200, "Temperature", 0, 1, "Deg");
            dialGaugeNode2.setOffset(300, 220);
            dialGaugeNode2.setValue(0.0);
            dialGaugeNode2.setElbowEnabled(true);
            dialGaugeNode2.setElbowHeight(200);
            testFrame.addNode(dialGaugeNode2);
            testFrame.setVisible(true);
        }
    }));
},
//------------------------------------------------------------------------
// Private Methods
//------------------------------------------------------------------------
 
  //private
updateConnector: function() {
    var width = (CONNECTOR_WIDTH_PROPORATION * m_diameter);
    var length = (CONNECTOR_LENGTH_PROPORATION * m_diameter);
    if (!m_elbowEnabled) {
        var connectorShape = new Rectangle.Number(0, 0, length, width);
        m_connector.setPathTo(connectorShape);
        m_connector.setOffset(m_dialComponentsNode.getFullBoundsReference().width * 0.9, m_diameter / 2 - width / 2);
    } else {
        m_connectorPath.reset();
        m_connectorPath.moveTo(0, 0);
        if (Math.abs(m_elbowHeight) < width / 2) {
            // width.
            m_connectorPath.lineTo(length + width, 0);
            m_connectorPath.lineTo((length + width), width);
            m_connectorPath.lineTo(0, width);
            m_connectorPath.closePath();
            m_connector.setPathTo(m_connectorPath);
        } else if (m_elbowHeight < 0) {
            // Connector is pointing upwards.
            m_connectorPath.lineTo(length, 0);
            m_connectorPath.lineTo(length, (m_elbowHeight + width / 2));
            m_connectorPath.lineTo((length + width), (m_elbowHeight + width / 2));
            m_connectorPath.lineTo((length + width), width / 2);
            m_connectorPath.quadTo(length + width, width, (length + (width / 2)), width);
            m_connectorPath.lineTo(0, width);
            m_connectorPath.closePath();
            m_connector.setPathTo(m_connectorPath);
        } else {
            // Connector is pointing downwards.
            m_connectorPath.lineTo((length + (width / 2)), 0);
            m_connectorPath.quadTo(length + width, 0, (length + width), width / 2);
            m_connectorPath.lineTo((length + width), (m_elbowHeight + width / 2));
            m_connectorPath.lineTo(length, (m_elbowHeight + width / 2));
            m_connectorPath.lineTo(length, width);
            m_connectorPath.lineTo(0, width);
            m_connectorPath.closePath();
            m_connector.setPathTo(m_connectorPath);
        }
    }
}
});
} );

